Package: simputation
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: GPL-3
Title: Simple Imputation
Type: Package
LazyLoad: yes
Authors@R: c( person("Mark", "van der Loo",
        role=c("aut","cre"),email="mark.vanderloo@gmail.com"))
Description: Easy to use interfaces to a number of imputation methods
        that fit in the not-a-pipe operator of the 'magrittr' package.
Version: 0.2.9
Depends: R (>= 4.0.0)
Imports: stats, utils, MASS, rpart, gower, VIM, randomForest, glmnet,
        missForest, norm
URL: https://github.com/markvanderloo/simputation
BugReports: https://github.com/markvanderloo/simputation/issues
Suggests: tinytest, knitr, rmarkdown, dplyr
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-12-16 15:32:14 UTC; mark
Author: Mark van der Loo [aut, cre]
Repository: CRAN
Date/Publication: 2024-12-16 16:10:02 UTC
Built: R 4.6.0; ; 2026-05-01 02:27:01 UTC; unix
