# Generated by roxygen2: do not edit by hand

S3method(foretell,default)
S3method(foretell,glm)
S3method(foretell,rpart)
S3method(print,simputation.capabilities)
export("%?>%")
export(foretell)
export(glimpse_na)
export(impute)
export(impute_cart)
export(impute_const)
export(impute_em)
export(impute_en)
export(impute_knn)
export(impute_lm)
export(impute_median)
export(impute_mf)
export(impute_pmm)
export(impute_proxy)
export(impute_rf)
export(impute_rhd)
export(impute_rlm)
export(impute_shd)
export(na.roughfix)
export(na.rpart)
export(na_status)
export(simputation_capabilities)
export(simputation_suggests)
import(stats)
importFrom(MASS,rlm)
importFrom(gower,gower_topn)
importFrom(rpart,na.rpart)
importFrom(rpart,prune)
importFrom(rpart,rpart)
importFrom(utils,installed.packages)
